﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Interactivity;
using System.Windows.Data;
using System.ComponentModel;
using System.Reflection;

namespace TreeViewMVVM.Helpers
{
    public class TreeViewBoundSelectedItemBehavior : Behavior<TreeView>
    {
        public static DependencyProperty TargetObjectProperty = DependencyProperty.Register(
                                                                    "TargetObject", 
                                                                    typeof(object), 
                                                                    typeof(TreeViewBoundSelectedItemBehavior),
                                                                    new PropertyMetadata(null, (s, e) => { ((TreeViewBoundSelectedItemBehavior)s).ResetBinding(e.OldValue, e.NewValue); }));

        private object _selectedObject;
        public object TargetObject
        {
            get { return base.GetValue(TargetObjectProperty); }
            set { base.SetValue(TargetObjectProperty, value); }
        }

        private string _propertyName;
        public string PropertyName
        {
            get { return _propertyName; }
            set
            {
                if (value != _propertyName)
                {
                    _propertyName = value;
                }
            }
        }

        protected override void OnAttached()
        {
            base.OnAttached();

            BindPropertyChanged(_selectedObject);
            AssociatedObject.SelectedItemChanged += new RoutedPropertyChangedEventHandler<object>(TreeView_SelectedItemChanged);
        }

        protected override void OnDetaching()
        {
            UnbindPropertyChanged(_selectedObject);
            AssociatedObject.SelectedItemChanged -= new RoutedPropertyChangedEventHandler<object>(TreeView_SelectedItemChanged);

            base.OnDetaching();
        }

        private void TreeView_SelectedItemChanged(object sender, RoutedPropertyChangedEventArgs<object> e)
        {
            SetSelectedItem(e.NewValue);
        }

        private bool _settingValue = false;
        private void OnPropertyChanged(object sender, PropertyChangedEventArgs args)
        {
            if (_settingValue) return;

            if (!String.IsNullOrEmpty(_propertyName))
            {
                if (StringComparer.InvariantCultureIgnoreCase.Compare(_propertyName, args.PropertyName) == 0)
                {
                    OnSelectedItemChanged(GetSelectedItem());
                }
            }
        }

        public object GetSelectedItem()
        {
            if (_selectedObject == null) return null;
            if (String.IsNullOrEmpty(_propertyName)) return null;

            PropertyInfo propInfo = _selectedObject.GetType().GetProperty(_propertyName, BindingFlags.Public | BindingFlags.IgnoreCase | BindingFlags.GetProperty | BindingFlags.Instance);
            if (propInfo == null) return null;

            return propInfo.GetValue(_selectedObject, null);
        }

        private void SetSelectedItem(object newItem)
        {
            _settingValue = true;

            try
            {
                if (_selectedObject == null) return;
                if (String.IsNullOrEmpty(_propertyName)) return;

                PropertyInfo propInfo = _selectedObject.GetType().GetProperty(_propertyName, BindingFlags.Public | BindingFlags.IgnoreCase | BindingFlags.SetProperty | BindingFlags.Instance);
                if (propInfo == null) return;

                propInfo.SetValue(_selectedObject, newItem, null);
            }
            finally
            {
                _settingValue = false;
            }
        }

        private void OnSelectedItemChanged(object newValue)
        {
            if (AssociatedObject == null)
            {
                return;
            }

            TreeViewItem item = AssociatedObject.ItemContainerGenerator.ContainerFromItem(newValue) as TreeViewItem;
            if (item != null)
            {
                item.IsSelected = true;
                return;
            }

            for (int i = 0; i < AssociatedObject.Items.Count; i++)
            {
                SelectItem(newValue, AssociatedObject.ItemContainerGenerator.ContainerFromIndex(i) as TreeViewItem);
            }
        }

        private static void SelectItem(object o, TreeViewItem parent)
        {
            if (parent == null)
            {
                return;
            }

            bool isExpanded = parent.IsExpanded;
            if (!isExpanded)
            {
                parent.IsExpanded = true;
                parent.UpdateLayout();
            }
            TreeViewItem item = parent.ItemContainerGenerator.ContainerFromItem(o) as TreeViewItem;
            if (item != null)
            {
                item.IsSelected = true;
                return;
            }

            for (int i = 0; i < parent.Items.Count; i++)
            {
                SelectItem(o, parent.ItemContainerGenerator.ContainerFromIndex(i) as TreeViewItem);
            }

            if (parent.IsExpanded != isExpanded)
            {
                parent.IsExpanded = isExpanded;
            }
        }

        private void ResetBinding(object oldValue, object newValue)
        {
            UnbindPropertyChanged(oldValue);
            _selectedObject = newValue;
            BindPropertyChanged(newValue);
        }

        private void UnbindPropertyChanged(object value)
        {
            if (value != null)
            {
                if (value is INotifyPropertyChanged)
                {
                    ((INotifyPropertyChanged)value).PropertyChanged -= OnPropertyChanged;
                }
            }
        }

        private void BindPropertyChanged(object value)
        {
            if (value != null)
            {
                if (value is INotifyPropertyChanged)
                {
                    ((INotifyPropertyChanged)value).PropertyChanged += OnPropertyChanged;
                }
            }
        }
    }
}
