﻿using System;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.ServiceModel.DomainServices.Client;
using TreeViewMVVM.Web.Models;
using System.Collections.ObjectModel;

namespace TreeViewMVVM
{
    public class MainPageViewModel : BaseViewModel
    {
        #region 'Child' View Models
        public class CategoryTreeViewItem : BaseViewModel
        {
            public CategoryTreeViewItem()
            {
                Children = new ObservableCollection<CategoryTreeViewItem>();
            }

            public CategoryTreeViewItem(Category category)
                : this()
            {
                Category = category;

                foreach (Category child in category.Children.OrderBy(i=> i.Sequence).ThenBy(i => i.Name))
                {
                    Children.Add(new CategoryTreeViewItem(child));
                }
            }

            public Category Category
            {
                get;
                set;
            }

            public ObservableCollection<CategoryTreeViewItem> Children { get; set; }

            public void RealignItemParentage()
            {
                foreach (CategoryTreeViewItem cat in Children)
                {
                    cat.Category.Sequence = (byte)Children.IndexOf(cat);
                    cat.Category.Parent = this.Category;

                    cat.RealignItemParentage();
                }
            }

        }
        #endregion

        #region Constructors
        public MainPageViewModel()
        {
            Categories = new ObservableCollection<CategoryTreeViewItem>();

            if (IsInDesignMode)
            {
                ConstructDesignTimeData();
            }
            else
            {
                CurrentContext.Load(
                    CurrentContext.GetCategoriesQuery(),
                    LoadBehavior.MergeIntoCurrent,
                    (loadOp) =>
                    {
                        if (!loadOp.HasError)
                        {
                            LoadCategories();
                        }
                        else
                        {
                            ShowMessageWindow("Error Retrieving Categories: " + loadOp.Error.Message);
                        }
                    },
                    null);
            }
        }
        #endregion

        #region Properties
        public ObservableCollection<CategoryTreeViewItem> Categories { get; set; }

        private CategoryTreeViewItem _selectedCategory;
        public CategoryTreeViewItem SelectedCategory
        {
            get { return _selectedCategory; }
            set
            {
                if (value != _selectedCategory)
                {
                    _selectedCategory = value;
                    OnPropertyChanged(() => SelectedCategory);
                }
            }
        }
        #endregion

        #region Public Methods
        public void RealignItemParentage()
        {
            foreach (CategoryTreeViewItem cat in Categories)
            {
                cat.Category.Sequence = Categories.IndexOf(cat);
                cat.Category.Parent = null;

                cat.RealignItemParentage();
            }

            Save();
        }

        public void Save()
        {
            CurrentContext.SubmitChanges();
        }

        public void AddCategory()
        {
            if (SelectedCategory == null)
            {
                // add at root
                CategoryTreeViewItem newItem = new CategoryTreeViewItem()
                {
                    Category = new Category()
                    {
                        CategoryId = Guid.NewGuid(),
                        Sequence = Categories.Count,
                        Name = "New Category"
                    }
                };

                Categories.Add(newItem);
                CurrentContext.Categories.Add(newItem.Category);
                SelectedCategory = newItem;
            }
            else
            {
                CategoryTreeViewItem newItem = new CategoryTreeViewItem()
                {
                    Category = new Category()
                    {
                        CategoryId = Guid.NewGuid(),
                        Sequence = SelectedCategory.Children.Count,
                        Name = "New Child Category",
                        Parent = SelectedCategory.Category
                    }
                };

                SelectedCategory.Children.Add(newItem);
                CurrentContext.Categories.Add(newItem.Category);
                SelectedCategory = newItem;
            }
        }
        #endregion

        #region Private Members
        private void LoadCategories()
        {
            foreach (Category rootCat in CurrentContext.Categories.Where(i => i.Parent == null).OrderBy(i => i.Sequence).ThenBy(i => i.Name))
            {
                Categories.Add(new CategoryTreeViewItem(rootCat));
            }
        }

        private void ConstructDesignTimeData()
        {
            Categories.Add(new CategoryTreeViewItem()
            {
                Category = new Category() { CategoryId = Guid.NewGuid(), Name = "Category 1" },
                Children = new ObservableCollection<CategoryTreeViewItem>() {
                    new CategoryTreeViewItem() { Category = new Category() { CategoryId = Guid.NewGuid(), Name= "Category 1.1"}},
                    new CategoryTreeViewItem() { Category = new Category() { CategoryId = Guid.NewGuid(), Name = "Category 1.2"}},
                    new CategoryTreeViewItem() { Category = new Category() { CategoryId = Guid.NewGuid(), Name = "Category 1.3"}},
                }
            });

            Categories.Add(new CategoryTreeViewItem()
            {
                Category = new Category() { CategoryId = Guid.NewGuid(), Name = "Category 2" },
                Children = new ObservableCollection<CategoryTreeViewItem>() {
                    new CategoryTreeViewItem() { Category = new Category() { CategoryId = Guid.NewGuid(), Name= "Category 2.1"}},
                    new CategoryTreeViewItem() { 
                        Category = new Category() { CategoryId = Guid.NewGuid(), Name = "Category 2.2"},
                        Children = new ObservableCollection<CategoryTreeViewItem>() {
                            new CategoryTreeViewItem() { Category = new Category() { CategoryId = Guid.NewGuid(), Name= "Category 2.2.1"}},
                            new CategoryTreeViewItem() { Category = new Category() { CategoryId = Guid.NewGuid(), Name= "Category 2.2.1"}},
                        }
                    },
                }
            });

            Categories.Add(new CategoryTreeViewItem()
            {
                Category = new Category() { CategoryId = Guid.NewGuid(), Name = "Category 3" },
                Children = new ObservableCollection<CategoryTreeViewItem>() {
                    new CategoryTreeViewItem() { Category = new Category() { CategoryId = Guid.NewGuid(), Name= "Category 3.1"}},
                }
            });

            foreach (CategoryTreeViewItem cat in Categories)
            {
                cat.RealignItemParentage();
            }
        }
        #endregion
    }
}
